% Multi compartment model of Hodgkin-Huxley model

%% Simulation

% Define global parameters ------------------------------------------------
global COMP;
global st_c;
global st_t;
% Model parameters --------------------------------------------------------
COMP = 100; % Number of compartment
st_c = 1;  % Number of inputed compartment
st_t = 0; % For simulation status
% Input stimulus ----------------------------------------------------------
stim = 0.0001; % [uA]
global Istim;
global last_t;
Istim  = [0 0; 0.999 0; 1 stim; 20 stim];
last_t = Istim(length(Istim), 1);
% Comp --------------------------------------------------------------------
tic % Start time
[t, y] = ode15s('hhmodel_multi');
fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b')
fprintf('Finished.       \n');
toc % Stop time

%% Plot data

% Color -------------------------------------------------------------------
ccol = colormap(hsv(COMP));
% Figure size
left = 60; bottom = 60; width = 600; height = 650;
pos    = get(gcf, 'Position');
pos(1) = left; pos(2) = bottom; pos(3) = width; pos(4) = height;
% Plotted graph
pcol = [];
% Plot --------------------------------------------------------------------
pl    = 0.1;  pwid1 = 0.8;
ppos1 = 0.74; ppos2 = 0.30; ppos3 = 0.08;
phei1 = 0.2;  phei2 = 0.42; phei3 = 0.09;
% Graph 1 (Stim) ==========================================================
subplot('Position', [pl ppos1 pwid1 phei1]);
plot([max(t) max(t)], [min(Istim(:,2))-max(Istim(:,2))*0.2 max(Istim(:,2))*1.2], 'k')
hold on
plot(Istim(:, 1), Istim(:, 2))
hold off
legend('location', 'NorthEast', 'Last time', 'Input stim')
ylabel('Input current [\muA]')
set(gca, 'Xticklabel', [])
axis([min(t) max(t)*1.42 min(Istim(:,2))-max(Istim(:,2))*0.2 max(Istim(:,2))*1.2] )
set(gcf, 'Position', pos)
% Graph 2 (Membrane potential) ============================================
subplot('Position', [pl ppos2 pwid1 phei2]);
base = plot([0 max(t)], [-65 -65]);
hold on
plot([max(t) max(t)], [-85 60], 'k')
for comp_l = 1:1:10
    comp = 1 + (comp_l - 1) * floor(COMP/10);
    str  = sprintf('fig = plot(t, y(:, %d));', 4 * (comp - 1) + 1);
    eval(str)
    set(fig, 'Color', ccol(comp, :))
    pcol = [pcol comp];
end
hold off
hAnnotation = get(base,'Annotation');
hLegendEntry = get(hAnnotation,'LegendInformation');
set(hLegendEntry,'IconDisplayStyle','off')
str = sprintf('legend(''location'',''NorthEast'',''comp%d'',''comp%d'',''comp%d'',''comp%d'',''comp%d'',''comp%d'',''comp%d'',''comp%d'',''comp%d'',''comp%d'')', ...
    pcol(1), pcol(2), pcol(3), pcol(4), pcol(5), pcol(6), pcol(7), pcol(8), pcol(9), pcol(10));
eval(str)
%xlabel('Time [msec]')
ylabel('Membrane potential [mV]')
axis([min(t) max(t)*1.42 -85 60])
xlabel('Time [msec]')

% Graph 3 (Movie) =========================================================
subplot('Position', [pl ppos3 pwid1 phei3]);
lw  = 10;
fig = plot([0 COMP], [0 0]);
%hold off
for time=1:1:size(t, 1)
    subplot('Position', [pl ppos3 pwid1 phei3]);
    for comp=1:1:COMP
        V = y(time, 4*(comp - 1) + 1);
        if V > 20
            plot([comp-1 comp], [0 0], '-', 'Color', [1 .5 0], 'LineWidth', lw);
        else
            plot([comp-1 comp], [0 0], '-', 'Color', 'b', 'LineWidth', lw);
        end
        if comp == 1
            hold on
        end
    end
    str = sprintf('title(''Time:%2.2f [msec]'', ''FontSize'', 20)', t(time));
    eval(str)
    xlabel('Compartment [No.]')
    if pcol(length(pcol)) == comp
        set(gca, 'Xtick', [pcol])
    else
        set(gca, 'Xtick', [pcol comp])
    end
    set(gca, 'Yticklabel', [])
    axis([0-COMP/10 COMP+COMP/10 -1 1])
    pause(0.01)
    hold off    
end